# -*- coding: utf-8 -*-

import json

from . import crypto
from .exceptions import AppNotFoundError, ImageNotFoundError
from .flock import lock_ex
from .paths import PUB_PRIVATE_KEY, PUB_REPO_FILE, PUB_REPO_LOCK, PUB_SIG_FILE

TYPE_APP = 'apps'
TYPE_IMAGE = 'images'

def load():
    try:
        with open(PUB_REPO_FILE) as f:
            return json.load(f)
    except FileNotFoundError:
        return {TYPE_IMAGE: {}, TYPE_APP: {}}

def save(data):
    with open(PUB_REPO_FILE, 'w') as f:
        json.dump(data, f, sort_keys=True, indent=4)
        # Cryptographically sign the repository file
        signature = crypto.sign_file(PUB_PRIVATE_KEY, PUB_REPO_FILE)
        with open(PUB_SIG_FILE, 'wb') as f:
            f.write(signature)

def get_entries(entry_type):
    with lock_ex(PUB_REPO_LOCK):
        data = load()
    return data[entry_type]

def get_entry(entry_type, name):
    return get_entries(entry_type)[name]

def add_entry(entry_type, name, definition):
    with lock_ex(PUB_REPO_LOCK):
        data = load()
        data[entry_type][name] = definition
        save(data)

def delete_entry(entry_type, name):
    with lock_ex(PUB_REPO_LOCK):
        data = load()
        try:
            del data[entry_type][name]
            save(data)
        except KeyError:
            pass

def get_images():
    return get_entries(TYPE_IMAGE)

def get_image(image_name):
    try:
        return get_entry(TYPE_IMAGE, image_name)
    except KeyError as e:
        raise ImageNotFoundError(image_name) from e

def register_image(image_name, definition):
    add_entry(TYPE_IMAGE, image_name, definition)

def unregister_image(image_name):
    delete_entry(TYPE_IMAGE, image_name)

def get_apps():
    return get_entries(TYPE_APP)

def get_app(app_name):
    try:
        return get_entry(TYPE_APP, image_name)
    except KeyError as e:
        raise ImageNotFoundError(image_name) from e

def register_app(app_name, definition):
    add_entry(TYPE_APP, image_name, definition)

def unregister_app(app_name):
    delete_entry(TYPE_APP, image_name)
